==> hermes-6.0.1.31.tar.gz: hermes-6.0.1.31/spec/libs/rex_client/tasks/update_contact_task_spec.rb <==
# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'
require [Hermes::Engine.root, %w[lib hermes rex_client tasks]].flatten.join('/').to_s
require [Hermes::Engine.root, %w[lib hermes rex_client]].flatten.join('/').to_s

HERMES_CONFIG['identity_provider_key'] = 'als_hermes_salarie'

RSpec.describe Hermes::RexClient::UpdateContactTask, type: :helper do
  5.times do |i|
    let!(:"entity#{i+1}") { create(:entity, id_cristal: "#{i+1}", siret: "0000000000000#{i+1}") }
  end 

  let!(:user) { create(:user, email: 'deborah.meliani@yopmail.com', password: Devise.friendly_token[0,20], group_ids: [entity2.group_id]) }
  let!(:user_right1) { create(:user_right, user: user, type: 'entity', entity_id: entity5.id, right: 'administrator') }
  let!(:rex_entity_deborah) do 
    {
      "fields" =>  {
        "nom" =>  "MELIANI",
        "rolecrelocal" =>  "CRE",
        "id_cristal" =>  "1",
        "prenom" =>  "DEBORAH",
        "email" =>  "deborah.meliani@yopmail.com",
        "id_etablissement_cristal" =>  [
          "1",
          "3",
          "4",
        ],
      },
    }
  end 

  describe 'Existing user' do
    subject { Hermes::RexClient::UpdateContactTask.new.update_contact(rex_entity_deborah) }

    it 'update user' do
      subject

      expect(user.reload.first_name).to eq "DEBORAH"
      expect(user.reload.auth_roles).to eq ["collaborateur offre logement", "cre"]
      expect(user.reload.id_cristal).to eq "1"
      expect(User.all().count).to eq(1)
    end

    it 'update access' do 
      expect(Hermes::Entity.writable_by(user).find_by(id: entity5.id)).not_to be_nil  

      subject

      user.reload
      expect(Hermes::Entity.writable_by(user).count).to eq(4)  
      expect(Hermes::Entity.writable_by(user).find_by(id: entity1.id)).not_to be_nil  
      expect(Hermes::Entity.writable_by(user).find_by(id: entity2.id)).to be_nil
      expect(Hermes::Entity.writable_by(user).find_by(id: entity3.id)).not_to be_nil  
      expect(Hermes::Entity.writable_by(user).find_by(id: entity4.id)).not_to be_nil  
      expect(Hermes::Entity.writable_by(user).find_by(id: entity5.id)).not_to be_nil  
    end 

    it 'update cre_email' do 
      entity5.set(updated_at: DateTime.current, cre_email: user.email)

      subject

      user.reload
      expect(entity1.reload.cre_email).to eq(user.email)
      expect(entity1.reload.cre_first_name).to eq("DEBORAH")
      expect(entity1.reload.cre_last_name).to eq("MELIANI")
      expect(entity2.reload.cre_email).to eq(nil)  
      expect(entity3.reload.cre_email).to eq(user.email)  
      expect(entity4.reload.cre_email).to eq(user.email)  
      expect(entity5.reload.cre_email).to eq(user.email)  
    end 

    it 'when already as right on entity' do
      ur2 = create(:user_right, user: user, type: 'entity', entity_id: entity1.id, right: 'reader')

      subject
      
      user.reload
      expect(Hermes::Entity.writable_by(user).find_by(id: entity1.id)).not_to be_nil  
      expect(entity1.reload.cre_email).to eq(user.email)
      expect(ur2.reload.right).to eq('administrator')
    end 
  end

  describe 'Not existing user' do
    it 'create user' do
      rex_entity = {
        "fields" =>  {
          "nom" =>  "Doe",
          "id_etablissement_cristal" =>  [
            "5",
          ],
          "rolecrelocal" =>  "CRE",
          "id_cristal" =>  "2",
          "prenom" =>  "John",
          "email" =>  "john.doe@yopmail.com"
        },
      }
      Hermes::RexClient::UpdateContactTask.new.update_contact(rex_entity)

      created_user = User.find_by(email: 'john.doe@yopmail.com')
      expect(User.all().count).to eq(2)
      expect(created_user.first_name).to eq "John"
      expect(Hermes::Entity.writable_by(user).count).to eq(1)  
      expect(Hermes::Entity.writable_by(created_user).find_by(id: entity5.id)).not_to be_nil
    end
  end
end

==> hermes-6.0.1.52.tar.gz: hermes-6.0.1.52/spec/libs/rex_client/tasks/update_contact_task_spec.rb <==
# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'
require [Hermes::Engine.root, %w[lib hermes rex_client tasks]].flatten.join('/').to_s
require [Hermes::Engine.root, %w[lib hermes rex_client]].flatten.join('/').to_s

HERMES_CONFIG['identity_provider_key'] = 'als_hermes_salarie'

RSpec.describe Hermes::RexClient::UpdateContactTask, type: :helper do
  5.times do |i|
    let!(:"entity#{i+1}") { create(:entity, id_cristal: "#{i+1}", siret: "0000000000000#{i+1}") }
  end 

  let!(:user) { create(:user, email: 'deborah.meliani@yopmail.com') }
  let!(:user_right1) { create(:user_right, user: user, type: 'entity', entity_id: entity5.id, right: 'administrator') }
  let!(:rex_entity_deborah) do 
    {
      "fields" =>  {
        "nom" =>  "MELIANI",
        "rolecrelocal" =>  "CRE",
        "id_cristal" =>  "1",
        "prenom" =>  "DEBORAH",
        "email" =>  "deborah.meliani@yopmail.com",
        "id_etablissement_cristal" =>  [
          "1",
          "3",
          "4",
        ],
      },
    }
  end 

  describe 'Existing user' do
    subject { Hermes::RexClient::UpdateContactTask.new.update_contact(rex_entity_deborah) }

    it 'update user' do
      subject

      expect(user.reload.first_name).to eq "DEBORAH"
      expect(user.reload.auth_roles).to eq ["collaborateur offre logement", "cre"]
      expect(user.reload.id_cristal).to eq "1"
      expect(User.all().count).to eq(1)
    end

    it 'update access' do 
      expect(Hermes::Entity.writable_by(user).find_by(id: entity5.id)).not_to be_nil  

      subject

      user.reload
      expect(Hermes::Entity.writable_by(user).count).to eq(4)  
      expect(Hermes::Entity.writable_by(user).find_by(id: entity1.id)).not_to be_nil  
      expect(Hermes::Entity.writable_by(user).find_by(id: entity2.id)).to be_nil
      expect(Hermes::Entity.writable_by(user).find_by(id: entity3.id)).not_to be_nil  
      expect(Hermes::Entity.writable_by(user).find_by(id: entity4.id)).not_to be_nil  
      expect(Hermes::Entity.writable_by(user).find_by(id: entity5.id)).not_to be_nil  
    end 

    it 'update cre_email' do 
      entity5.set(updated_at: DateTime.current, cre_email: user.email)

      subject

      user.reload
      expect(entity1.reload.cre_email).to eq(user.email)
      expect(entity1.reload.cre_first_name).to eq("DEBORAH")
      expect(entity1.reload.cre_last_name).to eq("MELIANI")
      expect(entity2.reload.cre_email).to eq(nil)  
      expect(entity3.reload.cre_email).to eq(user.email)  
      expect(entity4.reload.cre_email).to eq(user.email)  
      expect(entity5.reload.cre_email).to eq(nil)  
    end 

    it 'when already as right on entity' do
      ur2 = create(:user_right, user: user, type: 'entity', entity_id: entity1.id, right: 'reader')

      subject
      
      user.reload
      expect(Hermes::Entity.writable_by(user).find_by(id: entity1.id)).not_to be_nil  
      expect(entity1.reload.cre_email).to eq(user.email)
      expect(ur2.reload.right).to eq('administrator')
    end 
  end

  describe 'Not existing user' do
    it 'create user' do
      rex_entity = {
        "fields" =>  {
          "nom" =>  "Doe",
          "id_etablissement_cristal" =>  [
            "5",
          ],
          "rolecrelocal" =>  "CRE",
          "id_cristal" =>  "2",
          "prenom" =>  "John",
          "email" =>  "john.doe@yopmail.com"
        },
      }
      Hermes::RexClient::UpdateContactTask.new.update_contact(rex_entity)

      created_user = User.find_by(email: 'john.doe@yopmail.com')
      expect(User.all().count).to eq(2)
      expect(created_user.first_name).to eq "John"
      expect(Hermes::Entity.writable_by(user).count).to eq(1)  
      expect(Hermes::Entity.writable_by(created_user).find_by(id: entity5.id)).not_to be_nil
    end
  end

  describe 'not removing cre national' do
    let!(:user_right_national1) { create(:user_right, user_id: user.id, entity_id: entity1.id, right: 'administrator', type: 'entity', role: 'cre_national')}
    let!(:user_right_national3) { create(:user_right, user_id: user.id, entity_id: entity3.id, right: 'administrator', type: 'entity', role: 'cre_national')}

    it 'works' do
      user_right1.update(role: 'cre_national')
      entity5.update(cre_email: user.email)
      Hermes::RexClient::UpdateContactTask.new.update_contact(rex_entity_deborah)

      user.reload
      expect(Hermes::Entity.writable_by(user).count).to eq(4)  
      expect(Hermes::UserRight.find_by(user_id: user.id, entity_id: entity1.id).role).to eq('cre_national')
      expect(entity1.reload.cre_email).to eq(user.email)
      expect(entity5.reload.cre_email).to be_nil
    end 
  end 
end
